@@ -54,6 +54,8 @@ extern Rboolean AllDevicesKilled;
 #define DRAW(a) {drawing _d=xd->gawin;CLIP;a;NOBM(_d=xd->bm;CLIP;a;)}
 #define SHOW  gbitblt(xd->gawin,xd->bm,pt(0,0),getrect(xd->bm));
 
+#define SF 20  /* scrollbar resolution */
+
         /********************************************************/
 	/* Each driver can have its own device-specic graphical */
 	/* parameters and resources.  these should be wrapped	*/
@@ -404,7 +406,7 @@ static int SetBaseFont(gadesc *xd)
 /* 2 = oblique, 3 = bold-oblique */
 
 #define SMALLEST 2
-#define LARGEST 64
+#define LARGEST 100
 
 static void SetFont(int face, int size, double rot, DevDesc *dd)
 {
@@ -511,7 +513,7 @@ static void SetLinetype(int newlty, double nlwd, DevDesc *dd)
 /* Callback functions */
 
 
-static void HelpResize(window w,rect r)
+static void HelpResize(window w, rect r)
 {
     if (AllDevicesKilled) return;
     {
@@ -523,7 +525,7 @@ static void HelpResize(window w,rect r)
 		((xd->windowHeight != r.height))) {
 		xd->windowWidth = r.width;
 		xd->windowHeight = r.height;
-		xd->resize= TRUE;
+		xd->resize = TRUE;
 	    }
 	}
     }
@@ -1065,10 +1067,11 @@ static void devga_sbf(control c, int pos)
     DevDesc *dd = (DevDesc *) getdata(c);
     gadesc *xd = (gadesc *) dd->deviceSpecific;
     if (pos < 0) {
-	pos = min(-pos-1, xd->origWidth - xd->windowWidth);
+	pos = -pos-1;
+	pos = min(pos*SF, (xd->origWidth - xd->windowWidth + SF-1));
 	xd->xshift = -pos;
     } else {
-	pos = min(pos, xd->origHeight - xd->windowHeight);
+	pos = min(pos*SF, (xd->origHeight - xd->windowHeight + SF-1));
 	xd->yshift = -pos;
     }
     xd->resize = 1;
@@ -1087,15 +1090,20 @@ setupScreenDevice(DevDesc *dd, gadesc *xd, int w, int h,
     xd->kind = SCREEN;
     dw = dw0 = w / pixelWidth(NULL);
     dh = h / pixelHeight(NULL);
-    if ((dw / devicewidth(NULL)) > 0.85) {
-	d = dh / dw;
-	dw = 0.85 * devicewidth(NULL);
-	dh = d * dw;
-    }
-    if ((dh / deviceheight(NULL)) > 0.85) {
-	d = dw / dh;
-	dh = 0.85 * deviceheight(NULL);
-	dw = d * dh;
+    if (resize != 3) {
+	if ((dw / devicewidth(NULL)) > 0.85) {
+	    d = dh / dw;
+	    dw = 0.85 * devicewidth(NULL);
+	    dh = d * dw;
+	}
+	if ((dh / deviceheight(NULL)) > 0.85) {
+	    d = dw / dh;
+	    dh = 0.85 * deviceheight(NULL);
+	    dw = d * dh;
+	}
+    } else {
+	dw = min(dw, 0.85*devicewidth(NULL));
+	dh = min(dh, 0.85*deviceheight(NULL));
     }
     iw = dw + 0.5;
     ih = dh + 0.5;
@@ -1106,8 +1114,8 @@ setupScreenDevice(DevDesc *dd, gadesc *xd, int w, int h,
 				VScrollbar | HScrollbar))) {
 	return 0;
     }
-    gchangescrollbar(xd->gawin, VWINSB, 0, ih-1, ih, 0);
-    gchangescrollbar(xd->gawin, HWINSB, 0, iw-1, iw, 0);
+    gchangescrollbar(xd->gawin, VWINSB, 0, ih/SF-1, ih/SF, 0);
+    gchangescrollbar(xd->gawin, HWINSB, 0, iw/SF-1, iw/SF, 0);
 
     addto(xd->gawin);
     gsetcursor(xd->gawin, ArrowCursor);
@@ -1478,15 +1486,17 @@ static void GA_Resize(DevDesc *dd)
 	    dd->dp.left = dd->gp.left = shift;
 	    dd->dp.right = dd->gp.right = iw0 + shift;
 	    xd->xshift = shift;
-	    gchangescrollbar(xd->gawin, HWINSB, max(-shift,0), 
-			     xd->origWidth - 1, xd->windowWidth, 0);
+	    gchangescrollbar(xd->gawin, HWINSB, max(-shift,0)/SF, 
+			     xd->origWidth/SF - 1, xd->windowWidth/SF, 0);
 	    if(ih0 < ih) shift = (ih - ih0)/2.0; 
 	    else shift = min(0, xd->yshift);
 	    dd->dp.top = dd->gp.top = shift;
 	    dd->dp.bottom = dd->gp.bottom = ih0 + shift;
 	    xd->yshift = shift;
-	    gchangescrollbar(xd->gawin, VWINSB, max(-shift,0), 
-			     xd->origHeight - 1, xd->windowHeight, 0);
+	    gchangescrollbar(xd->gawin, VWINSB, max(-shift,0)/SF, 
+			     xd->origHeight/SF - 1, xd->windowHeight/SF, 0);
+	    xd->showWidth = xd->origWidth + min(0, xd->xshift);
+	    xd->showHeight = xd->origHeight + min(0,  xd->yshift);
 	}
 	xd->resize = FALSE;
 	if (xd->kind == SCREEN) {
@@ -1541,7 +1551,7 @@ static void GA_NewPage(DevDesc *dd)
 	DRAW(gfillrect(_d, xd->bgcolor, xd->clip));
     } else {
 	xd->clip = getregion(xd);
-	DRAW(gfillrect(_d, xd->bgcolor, getregion(xd)));
+	DRAW(gfillrect(_d, xd->bgcolor, xd->clip));
     }
 }
 
@@ -2030,6 +2040,12 @@ Rboolean GADeviceDriver(DevDesc *dd, char *display, double width,
     dd->dp.top = (xd->kind == PRINTER) ? rr.y : 0;	/* top */
     dd->dp.bottom = dd->dp.top + rr.height;	/* bottom */
 
+    if (resize == 3) { /* might have got a shrunken window */
+	int iw = width/pixelWidth(NULL) + 0.5, 
+	    ih = height/pixelHeight(NULL) + 0.5;
+	xd->origWidth = dd->dp.right = iw;
+	xd->origHeight = dd->dp.bottom = ih;
+    }
 
     /* Nominal Character Sizes in Pixels */
     gcharmetric(xd->gawin, xd->font, -1, &a, &d, &w);
@@ -2063,8 +2079,8 @@ Rboolean GADeviceDriver(DevDesc *dd, char *display, double width,
     /* initialise device description (most of the work */
     /* has been done in GA_Open) */
 
-    xd->resize= FALSE;
-    xd->locator= FALSE;
+    xd->resize = (resize == 3);
+    xd->locator = FALSE;
     dd->deviceSpecific = (void *) xd;
     dd->displayListOn = TRUE;
     if (RConsole && (xd->kind!=SCREEN)) show(RConsole);
