@@ -1,6 +1,6 @@
 system.time <- function(expr) {
     if(!exists("proc.time")) return(rep(NA, 5))
-    loc.frame <- sys.frame(sys.parent(1))
+    loc.frame <- parent.frame()
     on.exit(cat("Timing stopped at:", proc.time() - time, "\n"))
     expr <- substitute(expr)
     time <- proc.time()
