@@ -44,7 +44,7 @@ function(x, y, weights=rep(1,n), ww=rep(1,q), nterms, max.terms=nterms,
     if(!is.null(dimnames(x))) xnames <- dimnames(x)[[2]]
     else xnames <- paste("X", 1:p, sep="")
     if(!is.null(dimnames(y))) ynames <- dimnames(y)[[2]]
-    else ynames <- paste("Y", 1:p, sep="")
+    else ynames <- paste("Y", 1:q, sep="")
     msmod <- ml*(p+q+2*n)+q+7+ml+1	# for asr
     nsp <- n*(q+15)+q+3*p
     ndp <- p*(p+1)/2+6*p
