@@ -1,4 +1,4 @@
-#include <R_ext/Mathlib.h>
+#include <Rmath.h>
 #include <Rinternals.h>
 
 SEXP R_cutree(SEXP merge, SEXP which)
