@@ -27,6 +27,7 @@
 
 #define NIL -1
 #define ARGUSED(x) LEVELS(x)
+#define SET_ARGUSED(x,v) SETLEVELS(x,v)
 
 /* Hash function and equality test for keys */
 static int K, M;
@@ -48,40 +49,40 @@ static int scatter(unsigned int key)
     return 3141592653U * key >> (32 - K);
 }
 
-static int lhash(SEXP x, int index)
+static int lhash(SEXP x, int indx)
 {
-    if (LOGICAL(x)[index] == NA_LOGICAL)
+    if (LOGICAL(x)[indx] == NA_LOGICAL)
 	return 2;
-    return LOGICAL(x)[index];
+    return LOGICAL(x)[indx];
 }
 
-static int ihash(SEXP x, int index)
+static int ihash(SEXP x, int indx)
 {
-    if (INTEGER(x)[index] == NA_INTEGER)
+    if (INTEGER(x)[indx] == NA_INTEGER)
 	return 0;
-    return scatter((unsigned int) (INTEGER(x)[index]));
+    return scatter((unsigned int) (INTEGER(x)[indx]));
 }
 
-static int rhash(SEXP x, int index)
+static int rhash(SEXP x, int indx)
 {
     /* There is a problem with signed 0s under IEEE */
-    double tmp = (REAL(x)[index] == 0.0) ? 0.0 : REAL(x)[index];
+    double tmp = (REAL(x)[indx] == 0.0) ? 0.0 : REAL(x)[indx];
     return scatter(*((unsigned int *) (&tmp)));
 }
 
-static int chash(SEXP x, int index)
+static int chash(SEXP x, int indx)
 {
     Rcomplex tmp;
-    tmp.r = (COMPLEX(x)[index].r == 0.0) ? 0.0 : COMPLEX(x)[index].r;
-    tmp.i = (COMPLEX(x)[index].i == 0.0) ? 0.0 : COMPLEX(x)[index].i;
+    tmp.r = (COMPLEX(x)[indx].r == 0.0) ? 0.0 : COMPLEX(x)[indx].r;
+    tmp.i = (COMPLEX(x)[indx].i == 0.0) ? 0.0 : COMPLEX(x)[indx].i;
     return scatter((*((unsigned int *)(&tmp.r)) |
 		    (*((unsigned int *)(&tmp.r)))));
 }
 
-static int shash(SEXP x, int index)
+static int shash(SEXP x, int indx)
 {
     unsigned int k;
-    char *p = CHAR(STRING(x)[index]);
+    char *p = CHAR(STRING_ELT(x, indx));
     k = 0;
     while (*p++)
 	k = 8 * k + *p;
@@ -120,7 +121,7 @@ static int cequal(SEXP x, int i, SEXP y, int j)
 
 static int sequal(SEXP x, int i, SEXP y, int j)
 {
-    return !strcmp(CHAR(STRING(x)[i]), CHAR(STRING(y)[j]));
+    return !strcmp(CHAR(STRING_ELT(x, i)), CHAR(STRING_ELT(y, j)));
 }
 
 /* Choose M to be the smallest power of 2 */
@@ -172,18 +173,18 @@ static void HashTableSetup(SEXP x)
 /* Collision resolution is by linear probing */
 /* The table is guaranteed large so this is sufficient */
 
-static int isDuplicated(SEXP x, int index)
+static int isDuplicated(SEXP x, int indx)
 {
     int i, *h;
 
     h = INTEGER(HashTable);
-    i = hash(x, index);
+    i = hash(x, indx);
     while (h[i] != NIL) {
-	if (equal(x, h[i], x, index))
+	if (equal(x, h[i], x, indx))
 	    return 1;
 	i = (i + 1) % M;
     }
-    h[i] = index;
+    h[i] = indx;
     return 0;
 }
 
@@ -269,7 +270,7 @@ SEXP do_duplicated(SEXP call, SEXP op, SEXP args, SEXP env)
     case STRSXP:
 	for (i = 0; i < n; i++)
 	    if (LOGICAL(dup)[i] == 0)
-		STRING(ans)[k++] = STRING(x)[i];
+		SET_STRING_ELT(ans, k++, STRING_ELT(x, i));
 	break;
     }
     return ans;
@@ -290,14 +291,14 @@ static void DoHashing(SEXP table)
 	(void) isDuplicated(table, i);
 }
 
-static int Lookup(SEXP table, SEXP x, int index)
+static int Lookup(SEXP table, SEXP x, int indx)
 {
     int i, *h;
 
     h = INTEGER(HashTable);
-    i = hash(x, index);
+    i = hash(x, indx);
     while (h[i] != NIL) {
-	if (equal(table, h[i], x, index))
+	if (equal(table, h[i], x, indx))
 	    return h[i] + 1;
 	i = (i + 1) % M;
     }
@@ -330,13 +331,16 @@ SEXP do_match(SEXP call, SEXP op, SEXP args, SEXP env)
 	|| (!isVector(CADR(args)) && !isNull(CADR(args))))
 	error("match requires vector arguments");
 
-    /* Coerce to a common type */
-    /* Note: type == NILSXP is ok here */
-
-    type = TYPEOF(CAR(args)) < TYPEOF(CADR(args)) ?
-	TYPEOF(CADR(args)) : TYPEOF(CAR(args));
-    x = CAR(args) = coerceVector(CAR(args), type);
-    table = CADR(args) = coerceVector(CADR(args), type);
+    /* Coerce to a common type; type == NILSXP is ok here.  
+     * Note that R's match() does only coerce factors (to character).
+     * Hence, coerce to character or to `higher' type 
+     * (given that we have "Vector" or NULL) */
+    if(TYPEOF(CAR(args))  >= STRSXP || TYPEOF(CADR(args)) >= STRSXP)
+	type = STRSXP;
+    else type = TYPEOF(CAR(args)) < TYPEOF(CADR(args)) ?
+	     TYPEOF(CADR(args)) : TYPEOF(CAR(args));
+    x = SETCAR(args, coerceVector(CAR(args), type));
+    table = SETCADR(args, coerceVector(CADR(args), type));
     nomatch = asInteger(CAR(CDDR(args)));
     n = length(x);
 
@@ -405,14 +409,15 @@ SEXP do_pmatch(SEXP call, SEXP op, SEXP args, SEXP env)
     for (j = 0; j < n_target; j++) used[j] = 0;
     ans = allocVector(INTSXP, n_input);
     for (i = 0; i < n_input; i++) INTEGER(ans)[i] = 0;
- 
+
     /* First pass, exact matching */
     for (i = 0; i < n_input; i++) {
-	temp = strlen(CHAR(STRING(input)[i]));
+	temp = strlen(CHAR(STRING_ELT(input, i)));
 	if (temp == 0) continue;
 	for (j = 0; j < n_target; j++) {
 	    if (!dups_ok && used[j]) continue;
-	    k = strcmp(CHAR(STRING(input)[i]), CHAR(STRING(target)[j]));
+	    k = strcmp(CHAR(STRING_ELT(input, i)),
+		       CHAR(STRING_ELT(target, j)));
 	    if (k == 0) {
 		used[j] = 1;
 		INTEGER(ans)[i] = j + 1;
@@ -420,16 +425,17 @@ SEXP do_pmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 	    }
 	}
     }
-    /* Second pass, partial matching */   
+    /* Second pass, partial matching */
     for (i = 0; i < n_input; i++) {
 	if (INTEGER(ans)[i]) continue;
-	temp = strlen(CHAR(STRING(input)[i]));
+	temp = strlen(CHAR(STRING_ELT(input, i)));
 	if (temp == 0) continue;
 	mtch = 0;
 	mtch_count = 0;
 	for (j = 0; j < n_target; j++) {
 	    if (!dups_ok && used[j]) continue;
-	    k = strncmp(CHAR(STRING(input)[i]), CHAR(STRING(target)[j]), temp);
+	    k = strncmp(CHAR(STRING_ELT(input, i)),
+			CHAR(STRING_ELT(target, j)), temp);
 	    if (k == 0) {
 		mtch = j + 1;
 		mtch_count++;
@@ -443,17 +449,17 @@ SEXP do_pmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 
 #ifdef OLD_PMATCH
     for (i = 0; i < n_input; i++) {
-	temp = strlen(CHAR(STRING(input)[i]));
+	temp = strlen(CHAR(STRING_ELT(input, i)));
 	mtch = 0;
 	mtch_count = 0;
 	if (temp) {
 	    for (j = 0; j < n_target; j++) {
-		k = strncmp(CHAR(STRING(input)[i]),
-			    CHAR(STRING(target)[j]), temp);
+		k = strncmp(CHAR(STRING_ELT(input, i)),
+			    CHAR(STRING_ELT(target, j)), temp);
 		if (k == 0) {
 		    mtch = j + 1;
 		    if (dups_ok ||
-			strlen(CHAR(STRING(target)[j])) == temp)
+			strlen(CHAR(STRING_ELT(target, j))) == temp)
 			/* This is odd, effectively sets dups.ok
 			 * for perfect mtches, but that's what
 			 * Splus 3.4 does  --pd
@@ -478,7 +484,9 @@ SEXP do_pmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 SEXP do_charmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans, input, target;
-    int i, j, k, match, n_input, n_target, perfect, temp;
+    Rboolean perfect;
+    int i, j, k, imatch, n_input, n_target, temp;
+
     checkArity(op, args);
 
     input = CAR(args);
@@ -492,30 +500,30 @@ SEXP do_charmatch(SEXP call, SEXP op, SEXP args, SEXP env)
     ans = allocVector(INTSXP, n_input);
 
     for (i = 0; i < n_input; i++) {
-	temp = strlen(CHAR(STRING(input)[i]));
-	match = NA_INTEGER;
-	perfect = 0;
+	temp = strlen(CHAR(STRING_ELT(input, i)));
+	imatch = NA_INTEGER;
+	perfect = FALSE;
 	for (j = 0; j < n_target; j++) {
-	    k = strncmp(CHAR(STRING(input)[i]),
-			CHAR(STRING(target)[j]), temp);
+	    k = strncmp(CHAR(STRING_ELT(input, i)),
+			CHAR(STRING_ELT(target, j)), temp);
 	    if (k == 0) {
-		if (strlen(CHAR(STRING(target)[j])) == temp) {
-		    if (perfect == 1)
-			match = 0;
+		if (strlen(CHAR(STRING_ELT(target, j))) == temp) {
+		    if (perfect)
+			imatch = 0;
 		    else {
-			perfect = 1;
-			match = j + 1;
+			perfect = TRUE;
+			imatch = j + 1;
 		    }
 		}
-		else if (perfect == 0) {
-		    if (match == NA_INTEGER)
-			match = j + 1;
+		else if (!perfect) {
+		    if (imatch == NA_INTEGER)
+			imatch = j + 1;
 		    else
-			match = 0;
+			imatch = 0;
 		}
 	    }
 	}
-	INTEGER(ans)[i] = match;
+	INTEGER(ans)[i] = imatch;
     }
     return ans;
 }
@@ -538,7 +546,7 @@ static SEXP StripUnmatched(SEXP s)
 	return StripUnmatched(CDR(s));
     }
     else {
-	CDR(s) = StripUnmatched(CDR(s));
+	SETCDR(s, StripUnmatched(CDR(s)));
 	return s;
     }
 }
@@ -549,21 +557,21 @@ static SEXP ExpandDots(SEXP s, int expdots)
     if (s == R_NilValue)
 	return s;
     if (TYPEOF(CAR(s)) == DOTSXP ) {
-	TYPEOF(CAR(s)) = LISTSXP;	/* a safe mutation */
+	SET_TYPEOF(CAR(s), LISTSXP);	/* a safe mutation */
 	if (expdots) {
 	    r = CAR(s);
 	    while (CDR(r) != R_NilValue ) {
-		ARGUSED(r) = 1;
+		SET_ARGUSED(r, 1);
 		r = CDR(r);
 	    }
-	    ARGUSED(r) = 1;
-	    CDR(r)= ExpandDots(CDR(s), expdots);
+	    SET_ARGUSED(r, 1);
+	    SETCDR(r, ExpandDots(CDR(s), expdots));
 	    return CAR(s);
 	}
     }
     else
-	ARGUSED(s) = 0;
-    CDR(s) = ExpandDots(CDR(s), expdots);
+	SET_ARGUSED(s, 0);
+    SETCDR(s, ExpandDots(CDR(s), expdots));
     return s;
 }
 static SEXP subDots(SEXP rho)
@@ -585,14 +593,14 @@ static SEXP subDots(SEXP rho)
     for(a=dots, b=rval, i=1; i<=len; a=CDR(a), b=CDR(b), i++) {
 	sprintf(tbuf,"..%d",i);
 	if( TAG(a) != R_NilValue )
-	    TAG(b) = TAG(a);
+	    SET_TAG(b, TAG(a));
 	else
-	    TAG(b) = install(tbuf);
+	    SET_TAG(b, install(tbuf));
 	if( isSymbol(PREXPR(CAR(a))) || isLanguage(PREXPR(CAR(a))) ) {
-		CAR(b) = mkSYMSXP(mkChar(tbuf), R_UnboundValue);
+		SETCAR(b, mkSYMSXP(mkChar(tbuf), R_UnboundValue));
 	}
 	else
-		CAR(b) = PREXPR(CAR(a));
+		SETCAR(b, PREXPR(CAR(a)));
     }
     UNPROTECT(1);
     return rval;
@@ -611,11 +619,11 @@ SEXP do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
     funcall = CADR(args);
 
     if (TYPEOF(funcall) == EXPRSXP)
-	funcall = VECTOR(funcall)[0];
+	funcall = VECTOR_ELT(funcall, 0);
 
     if (TYPEOF(funcall) != LANGSXP) {
 	b = deparse1(funcall, 1);
-	errorcall(call, "%s is not a valid call", CHAR(STRING(b)[0]));
+	errorcall(call, "%s is not a valid call", CHAR(STRING_ELT(b, 0)));
     }
 
     /* Get the function definition */
@@ -645,7 +653,7 @@ SEXP do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
 
     if (TYPEOF(b) != CLOSXP) {
 	b = deparse1(b, 1);
-	errorcall(call, "%s is not a function", CHAR(STRING(b)[0]));
+	errorcall(call, "%s is not a function", CHAR(STRING_ELT(b, 0)));
     }
 
     /* Do we expand ... ? */
@@ -653,7 +661,7 @@ SEXP do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
     expdots = asLogical(CAR(CDDR(args)));
     if (expdots == NA_LOGICAL) {
 	b = deparse1(CADDR(args), 1);
-	errorcall(call, "%s is not a logical", CHAR(STRING(b)[0]));
+	errorcall(call, "%s is not a logical", CHAR(STRING_ELT(b, 0)));
     }
 
     /* Get the formals and match the actual args */
@@ -682,13 +690,16 @@ SEXP do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
     }
     /* now to splice t2 into the correct spot in actuals */
     if (t2 != R_MissingArg ) {	/* so we did something above */
-	if( CAR(actuals) == R_DotsSymbol )
-		actuals = listAppend(t2, CDR(actuals));
+	if( CAR(actuals) == R_DotsSymbol ) {
+	        UNPROTECT(1);
+	        actuals = listAppend(t2, CDR(actuals));
+	        PROTECT(actuals);
+	}
 	else {
 		for(t1=actuals; t1!=R_NilValue; t1=CDR(t1)) {
 			if( CADR(t1) == R_DotsSymbol ) {
 				tail = CDDR(t1);
-				CDR(t1) = t2;
+				SETCDR(t1, t2);
 				listAppend(actuals,tail);
 				break;
 			}
@@ -701,7 +712,7 @@ SEXP do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
     /* Attach the argument names as tags */
 
     for (f = formals, b = rlist; b != R_NilValue; b = CDR(b), f = CDR(f)) {
-	TAG(b) = TAG(f);
+	SET_TAG(b, TAG(f));
     }
 
 
@@ -715,8 +726,12 @@ SEXP do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
     rlist = StripUnmatched(rlist);
 
     PROTECT(rval = allocSExp(LANGSXP));
-    CAR(rval) = duplicate(CAR(funcall));
-    CDR(rval) = rlist;
+    SETCAR(rval, duplicate(CAR(funcall)));
+    SETCDR(rval, rlist);
     UNPROTECT(4);
     return rval;
 }
+
+
+
+
